/*******************************************************************
*  CHECKER.CXX
*  (c) 1992-1994 STAR DIVISION
*******************************************************************/

#include <sv.hxx>

#define SQUARES 5

// --- class MyApp -------------------------------------------------

class MyApp : public Application
{
public:
    virtual void Main( int, char*[] );
};

// --- class CheckWindow -------------------------------------------

class CheckWindow : public WorkWindow
{
protected:
    USHORT          nSizeX;
    USHORT          nSizeY;
    BOOL            bChecked[SQUARES][SQUARES];

public:
                    CheckWindow();

    virtual void    Paint( const Rectangle& );
    virtual void    Resize();
    virtual void    MouseButtonDown( const MouseEvent& rMEvt );
    virtual void    KeyInput( const KeyEvent& rKEvt );
};

// --- CheckWindow::CheckWindow() ----------------------------------

CheckWindow::CheckWindow() :
                 WorkWindow( NULL, WB_APP | WB_STDWORK )
{
    nSizeX = GetOutputSizePixel().Width() / SQUARES;
    nSizeY = GetOutputSizePixel().Height() / SQUARES;

    for ( USHORT i = 0; i < SQUARES; i++ )
        for ( USHORT j = 0; j < SQUARES; j++ )
            bChecked[i][j] = FALSE;

    SetText( "Checker 2" );
    Show();
}

// --- CheckWindow::Resize() ---------------------------------------

void CheckWindow::Resize()
{
    nSizeX = GetOutputSizePixel().Width() / SQUARES;
    nSizeY = GetOutputSizePixel().Height() / SQUARES;
    Invalidate();
}

// --- CheckWindow::Paint() ----------------------------------------

void CheckWindow::Paint( const Rectangle& )
{
    for ( short x = 0; x < SQUARES; x++)
    {
        for ( short y = 0; y < SQUARES; y++)
        {
            DrawRect( Rectangle( Point( x*nSizeX, y*nSizeY ),
                                 Size( nSizeX, nSizeY ) ) );

            if ( bChecked[x][y] )
            {
                DrawLine ( Point( x*nSizeX+1,
                                  y*nSizeY+1 ),
                           Point( x*nSizeX+nSizeX-2,
                                  y*nSizeY+nSizeY-2 ) );
                DrawLine ( Point( x*nSizeX+nSizeX-2,
                                  y*nSizeY+2 ),
                           Point( x*nSizeX+1,
                                  y*nSizeY+nSizeY-2 ) );
            }
        }
    }
}

// --- CheckWindow::MouseButtonDown() ------------------------------

void CheckWindow::MouseButtonDown( const MouseEvent& rMEvt )
{
    USHORT x = rMEvt.GetPosPixel().X() / nSizeX;
    USHORT y = rMEvt.GetPosPixel().Y() / nSizeY;

    if ( (x < SQUARES) && (y < SQUARES) )
    {
        bChecked[x][y] = !bChecked[x][y];

        if ( bChecked[x][y] )
            ChangePen( Pen( Color( COL_BLACK ) ) );
        else
            ChangePen( Pen( Color( COL_WINDOWWORKSPACE ) ) );

        DrawLine ( Point( x*nSizeX+1, y*nSizeY+1 ),
                   Point( x*nSizeX+nSizeX-2, y*nSizeY+nSizeY-2 ) );
        DrawLine ( Point( x*nSizeX+nSizeX-2, y*nSizeY+2 ),
                   Point( x*nSizeX+1, y*nSizeY+nSizeY-2 ) );

        ChangePen( Pen( Color( COL_BLACK ) ) );
    }
}

// --- CheckWindow::KeyInput() -------------------------------------

void CheckWindow::KeyInput( const KeyEvent& rKEvt)
{
    Point aOutPos( ScreenToOutputPixel( Pointer::GetPosPixel() ) );
    short x = aOutPos.X() / nSizeX;
    short y = aOutPos.Y() / nSizeY;

    if ( x < 0 )
        x = 0;
    else if ( x >= SQUARES )
        x = SQUARES - 1;

    if ( y < 0 )
        y = 0;
    else if ( y >= SQUARES )
        y = SQUARES - 1;

    switch( rKEvt.GetKeyCode().GetCode() )
    {
        case KEY_DOWN:
            y++;
            break;
        case KEY_UP:
            y--;
            break;
        case KEY_RIGHT:
            x++;
            break;
        case KEY_LEFT:
            x--;
            break;
        case KEY_HOME:
            x = y = 0;
            break;
        case KEY_END:
            x = y = ( SQUARES - 1 );
            break;
        case KEY_SPACE:
        case KEY_RETURN:
            MouseButtonDown( MouseEvent( aOutPos ) );
            return;
        default:
            WorkWindow::KeyInput( rKEvt );
            return;
    }

    x = (((x+SQUARES) % SQUARES) * nSizeX) + nSizeX/2;
    y = (((y+SQUARES) % SQUARES) * nSizeY) + nSizeY/2;

    Pointer::ChangePosPixel( OutputToScreenPixel( Point( x, y ) ) );
}

// --- MyApp:Main() ------------------------------------------------

void MyApp::Main( int, char*[] )
{
    CheckWindow aWindow;
    Execute();
}

// --- aMyApp ------------------------------------------------------

MyApp aMyApp;
