/*******************************************************************
*  MENUDEMO.CXX
*  (c) 1992-1994 STAR DIVISION
*******************************************************************/

#include <sv.hxx>

#include "menudemo.hrc"

// --- class MyApp -------------------------------------------------

class MyApp : public Application
{
public:
    virtual void Main( int, char*[] );
};

// --- class MenuWindow --------------------------------------------

class MenuWindow : public WorkWindow
{
protected:
    USHORT          nLastColor;
    AutoTimer       aTimer;

public:
                    MenuWindow();

    virtual void    MouseButtonDown( const MouseEvent& rMEvt );
    virtual void    Paint( const Rectangle& );

    BOOL            DoCommand( USHORT nItemId );
    long            MenuSelectHdl( Menu* pMenu );
    long            AccelSelectHdl( Accelerator* pAccel );

    void            TimeBeep( Timer* ) { Sound::Beep(); }
};

// --- MenuWindow::MenuWindow() ------------------------------------

MenuWindow::MenuWindow() :
                WorkWindow( NULL, WB_APP | WB_STDWORK )
{
    nLastColor = BM_WHITE;

    aTimer.ChangeTimeoutHdl( LINK( this, MenuWindow, TimeBeep ) );
    aTimer.ChangeTimeout( 1000 );

    SetText( "MenuDemo" );
}

// --- MenuWindow::MouseButtonDown() -------------------------------

void MenuWindow::MouseButtonDown( const MouseEvent& rMEvt )
{
    pApp->GetAppMenu()->GetPopupMenu( MB_BACKGROUND )->
        Execute( OutputToScreenPixel( rMEvt.GetPosPixel() ) );
}

// --- MenuWindow::Paint() -----------------------------------------

void MenuWindow::Paint( const Rectangle& )
{
    DrawText( Point( 10, 10 ),
              "Press a MouseButton for a PopupMenu" );

    String aKeyText = "Press ";
    aKeyText += KeyCode( KEY_MOD1 ).GetName();
    aKeyText += " and 'W', 'L', 'G' or 'B' for Colors";
    DrawText( Point( 10, 40 ), aKeyText );
}

// --- MenuWindow::DoCommand() -------------------------------------

BOOL MenuWindow::DoCommand( USHORT nItemId )
{
    Menu*       pMenu;
    ColorName   eColor;

    pMenu = pApp->GetAppMenu()->GetPopupMenu( MB_BACKGROUND );

    switch ( nItemId )
    {
        case BM_WHITE:
        case BM_LTGRAY:
        case BM_GRAY:
        case BM_BLACK:
            {
            pMenu->CheckItem( nLastColor, FALSE );
            nLastColor = nItemId;
            switch( nLastColor )
            {
                case BM_WHITE:
                    eColor = COL_WHITE;
                    break;
                case BM_LTGRAY:
                    eColor = COL_LIGHTGRAY;
                    break;
                case BM_GRAY:
                    eColor = COL_GRAY;
                    break;
                case BM_BLACK:
                    eColor = COL_BLACK;
                    break;
            }
            ChangeBackgroundBrush( Brush( Color( eColor ) ) );
            pMenu->CheckItem( nLastColor );
            Invalidate();
            }
            break;

        default:
            return FALSE;
    }

    return TRUE;
}

// --- MenuWindow::MenuSelectHdl() ---------------------------------

long MenuWindow::MenuSelectHdl( Menu* pMenu )
{
    USHORT nItemId = pMenu->GetCurItemId();

    switch ( nItemId )
    {
        case FM_NEW:
        case FM_OPEN:
        case FM_SAVE:
        case FM_SAVEAS:
        case EM_UNDO:
        case EM_CUT:
        case EM_COPY:
        case EM_PASTE:
        case EM_CLEAR:
            {
                InfoBox( this, pMenu->GetItemText(
                               pMenu->GetCurItemId() ) ).Execute();
            }
            break;

        case FM_EXIT:
            pApp->Quit();
            break;

        case TM_START:
            aTimer.Start();
            pMenu->EnableItem( nItemId, FALSE );
            pMenu->EnableItem( TM_STOP, TRUE );
            break;
        case TM_STOP:
            aTimer.Stop();
            pMenu->EnableItem( nItemId, FALSE );
            pMenu->EnableItem( TM_START, TRUE );
            break;

        case HM_HELP:
            {
                InfoBox aHelpBox( this, "No Help available." );
                aHelpBox.Execute();
            }
            break;
        case HM_ABOUT:
            {
                InfoBox aAboutBox( this, "MenuDemo\n"
                                         "(c) STAR DIVISION" );
                aAboutBox.Execute();
            }
            break;

        default:
            return DoCommand( pMenu->GetCurItemId() );
    }

    return TRUE;

}

// --- MenuWindow::AccelSelectHdl() --------------------------------

long MenuWindow::AccelSelectHdl( Accelerator* pAccel )
{
    return DoCommand( pAccel->GetCurItemId() );
}

// --- MyApp::Main() -----------------------------------------------

void MyApp::Main( int, char*[] )
{
    MenuBar         aAppMenu( ResId( APP_MB ) );
    Accelerator     aAppAccel( ResId( APP_ACCEL ) );
    MenuWindow      aMenuWindow;

    aAppMenu.PushSelectHdl( LINK( &aMenuWindow,
                                  MenuWindow, MenuSelectHdl ) );
    aAppMenu.GetPopupMenu( MB_BACKGROUND )->
                PushSelectHdl( LINK( &aMenuWindow,
                                     MenuWindow, MenuSelectHdl ) );
    ChangeAppMenu( &aAppMenu );

    aAppAccel.PushSelectHdl( LINK( &aMenuWindow,
                                   MenuWindow, AccelSelectHdl ) );
    InsertAccel( &aAppAccel );

    aMenuWindow.Show();

    Execute();
}

// --- aMyApp ------------------------------------------------------

MyApp aMyApp;
