/*******************************************************************
*  FONTDEMO.CXX
*  (c) 1992-1994 STAR DIVISION
*******************************************************************/

#include <sv.hxx>

#include "fontdemo.hrc"

// --- class MyApp -------------------------------------------------

class MyApp : public Application
{
public:
    virtual void Main( int, char*[] );
};

// --- class ShowFont ----------------------------------------------

class ShowFont : public Control
{
public:
                    ShowFont( Window* pParent,
                              const ResId& rResId );

    virtual void    Paint( const Rectangle& );
    Font            ChangeFont( const Font& rFont );
};

// --- class MyFontDialog ------------------------------------------

class MyFontDialog : public ModalDialog
{
protected:
    FixedText           aNameText;
    ListBox             aNameBox;
    FixedText           aSizeText;
    ListBox             aSizeBox;
    CheckBox            aItalic;
    CheckBox            aBold;
    CheckBox            aStrikeout;
    CheckBox            aUnderline;
    CheckBox            aShadow;
    CheckBox            aOutline;
    CheckBox            aTransparent;
    GroupBox            aStyleGroup;
    FixedText           aColorText;
    ListBox             aColorBox;
    FixedText           aFillText;
    ListBox             aFillBox;
    FixedText           aLineAngleText;
    ComboBox            aLineAngleBox;
    FixedText           aCharAngleText;
    ComboBox            aCharAngleBox;
    ShowFont            aShowFont;
    OKButton            aOKButton;
    Font                aFont;

public:
                        MyFontDialog( Window* pWindow );

    void                FontChangeHdl( ListBox* pBox );
    void                SizeChangeHdl( ListBox* pBox );

    void                ColorSelectHdl( ListBox* pBox );
    void                AttrHdl( CheckBox* pBox );

    void                LineSelectHdl( ComboBox* pBox );
    void                LineModifyHdl( ComboBox* pBox );
    void                CharSelectHdl( ComboBox* pBox );
    void                CharModifyHdl( ComboBox* pBox );

    Font                GetNewFont() const { return aFont; }
};

// --- class AppWin ------------------------------------------------

class AppWin : public WorkWindow
{
private:
    MyFontDialog    aFontDialog;
    PushButton      aFontButton;

public:
                    AppWin();

    virtual void    Paint( const Rectangle& );
    void            FontHdl( PushButton* );
};

// --- ShowFont::ShowFont() ----------------------------------------

ShowFont::ShowFont( Window* pParent, const ResId& rResId ) :
              Control( pParent, rResId )
{
    ChangeBackgroundBrush( Brush( Color( COL_WHITE ) ) );
}

// --- ShowFont::Paint() -------------------------------------------

void ShowFont::Paint( const Rectangle& )
{
    USHORT x,y;
    Size   aTextSize( GetTextSize( GetText() ) );
    Size   aWindowSize( GetOutputSize() );

    if ( GetFont().GetLineOrientation() != 0 )
    {
        x = aWindowSize.Width()/2;
        y = aWindowSize.Height()/2;
        DrawText( Point( x, y ), "Rotation" );
    }
    else
    {
        x = aWindowSize.Width()/2 - aTextSize.Width()/2;
        y = aWindowSize.Height()/2 - aTextSize.Height()/2;
        DrawText( Point( x, y ), GetText() );
    }
}

// --- ShowFont::ChangeFont() --------------------------------------

Font ShowFont::ChangeFont( const Font& rFont )
{
    Invalidate();
    return Control::ChangeFont( rFont );
}

// --- MyFontDialog::MyFontDialog() --------------------------------

MyFontDialog::MyFontDialog( Window* pWindow ) :
                ModalDialog( pWindow, ResId( FD_FONTDIALOG ) ),
                aNameText( this, ResId( FD_NAMETEXT ) ),
                aNameBox( this, ResId( FD_NAMEBOX ) ),
                aSizeText( this, ResId( FD_SIZETEXT ) ),
                aSizeBox( this, ResId( FD_SIZEBOX ) ),
                aItalic( this, ResId( FD_ITALIC ) ),
                aBold( this, ResId( FD_BOLD ) ),
                aStrikeout( this, ResId( FD_STRIKEOUT ) ),
                aUnderline( this, ResId( FD_UNDERLINE ) ),
                aShadow( this, ResId( FD_SHADOW ) ),
                aOutline( this, ResId( FD_OUTLINE ) ),
                aTransparent( this, ResId( FD_TRANSPARENT ) ),
                aStyleGroup( this, ResId( FD_STYLEGROUP ) ),
                aColorText( this, ResId( FD_COLORTEXT ) ),
                aColorBox( this, ResId( FD_COLORBOX ) ),
                aFillText( this, ResId( FD_FILLINTEXT ) ),
                aFillBox( this, ResId( FD_FILLINBOX ) ),
                aLineAngleText( this, ResId( FD_LINEANGLETEXT ) ),
                aLineAngleBox( this, ResId( FD_LINEANGLEBOX ) ),
                aCharAngleText( this, ResId( FD_CHARANGLETEXT ) ),
                aCharAngleBox( this, ResId( FD_CHARANGLEBOX ) ),
                aShowFont( this, ResId( FD_SHOWFONT ) ),
                aOKButton( this, ResId( FD_OK ) )
{
    FreeResource();

    int n = GetDevFontCount();
    for ( int i = 0; i < n; i++ )
        aNameBox.InsertEntry( GetDevFont( i ).GetName() );
    aNameBox.SelectEntryPos( 0 );

    aNameBox.ChangeSelectHdl(
                 LINK( this, MyFontDialog, FontChangeHdl ) );
    aSizeBox.ChangeSelectHdl(
                 LINK( this, MyFontDialog, SizeChangeHdl ) );
    aItalic.ChangeClickHdl( LINK( this, MyFontDialog, AttrHdl ) );
    aBold.ChangeClickHdl( LINK( this, MyFontDialog, AttrHdl ) );
    aStrikeout.ChangeClickHdl( LINK( this, MyFontDialog,
                                     AttrHdl ) );
    aUnderline.ChangeClickHdl( LINK( this, MyFontDialog,
                                     AttrHdl ) );
    aShadow.ChangeClickHdl( LINK( this, MyFontDialog, AttrHdl ) );
    aOutline.ChangeClickHdl( LINK( this, MyFontDialog, AttrHdl ) );
    aTransparent.ChangeClickHdl(
                     LINK( this, MyFontDialog, AttrHdl ) );
    aColorBox.ChangeSelectHdl(
                  LINK( this, MyFontDialog, ColorSelectHdl ) );
    aFillBox.ChangeSelectHdl(
                 LINK( this, MyFontDialog, ColorSelectHdl ) );
    aLineAngleBox.ChangeModifyHdl(
                      LINK( this, MyFontDialog, LineModifyHdl ) );
    aLineAngleBox.ChangeSelectHdl(
                      LINK( this, MyFontDialog, LineSelectHdl ) );
    aCharAngleBox.ChangeModifyHdl(
                      LINK( this, MyFontDialog, CharModifyHdl ) );
    aCharAngleBox.ChangeSelectHdl(
                      LINK( this, MyFontDialog, CharSelectHdl ) );

    FontChangeHdl( &aNameBox );
}

// --- MyFontDialog::FontChangeHdl() -------------------------------

void MyFontDialog::FontChangeHdl( ListBox* pBox )
{
    aFont.ChangeName( pBox->GetSelectEntry() );
    String aOldSize = aSizeBox.GetSelectEntry();
    aSizeBox.ChangeUpdateMode( FALSE );
    aSizeBox.Clear();
    USHORT nHeight = GetDevFontSize( aFont, 0 ).Height();
    if ( !nHeight )
    {
        for ( int i = 1; i < 128; i++ )
            aSizeBox.InsertEntry( i );
    }
    else
    {
        int n = GetDevFontSizeCount( aFont );
        for ( int i = 0; i < n; i++ )
            aSizeBox.InsertEntry( GetDevFontSize( aFont,
                                                  i ).Height() );
    }
    aSizeBox.SelectEntry( aOldSize );
    aSizeBox.ChangeUpdateMode( TRUE );
    aSizeBox.Invalidate();
    SizeChangeHdl( &aSizeBox );
}

// --- MyFontDialog::SizeChangeHdl() -------------------------------

void MyFontDialog::SizeChangeHdl( ListBox* pBox )
{
    aFont.ChangeSize( Size( 0, pBox->GetSelectEntry() ) );
    aShowFont.ChangeFont( aFont );
}

// --- MyFontDialog::ColorSelectHdl() ------------------------------

void MyFontDialog::ColorSelectHdl( ListBox* pBox )
{
    ColorName eColorName;

    switch ( pBox->GetSelectEntryPos() )
    {
        case 0:
            eColorName = COL_BLACK;
            break;
        case 1:
            eColorName = COL_RED;
            break;
        case 2:
            eColorName = COL_GREEN;
            break;
        case 3:
            eColorName = COL_BLUE;
            break;
        case 4:
            eColorName = COL_LIGHTRED;
            break;
        case 5:
            eColorName = COL_LIGHTGREEN;
            break;
        case 6:
            eColorName = COL_LIGHTBLUE;
            break;
        case 7:
            eColorName = COL_YELLOW;
            break;
        case 8:
            eColorName = COL_WHITE;
            break;
    }

    if ( pBox == &aColorBox )
        aFont.ChangeColor( Color( eColorName ) );
    else
        aFont.ChangeFillColor( Color( eColorName ) );

    aShowFont.ChangeFont( aFont );
}

// --- MyFontDialog::AttrHdl() -------------------------------------

void MyFontDialog::AttrHdl( CheckBox* )
{
    if ( aBold.IsChecked() )
        aFont.ChangeWeight( FontWeight( WEIGHT_BOLD ) );
    else
        aFont.ChangeWeight( FontWeight( WEIGHT_NORMAL ) );

    aFont.ChangeItalic( aItalic.IsChecked() );
    if ( aStrikeout.IsChecked() )
        aFont.ChangeStrikeout( STRIKEOUT_SINGLE );
    else
        aFont.ChangeStrikeout( STRIKEOUT_NONE );
    if ( aUnderline.IsChecked() )
        aFont.ChangeUnderline( UNDERLINE_SINGLE );
    else
        aFont.ChangeUnderline( UNDERLINE_NONE );
    aFont.ChangeShadow( aShadow.IsChecked());
    aFont.ChangeOutline( aOutline.IsChecked() );
    aFont.ChangeTransparent( aTransparent.IsChecked() );

    aShowFont.ChangeFont( aFont );
}

// --- MyFontDialog::LineSelectHdl() -------------------------------

void MyFontDialog::LineSelectHdl( ComboBox* pBox )
{
    aFont.ChangeLineOrientation( (short)pBox->GetText() );
    aShowFont.ChangeFont( aFont );
}

// --- MyFontDialog::LineModifyHdl() -------------------------------

void MyFontDialog::LineModifyHdl( ComboBox* pBox )
{
    aFont.ChangeLineOrientation( pBox->GetText() );
    aShowFont.ChangeFont( aFont );
}

// --- MyFontDialog::CharSelectHdl() -------------------------------

void MyFontDialog::CharSelectHdl( ComboBox* pBox )
{
    aFont.ChangeCharOrientation( pBox->GetText() );
    aShowFont.ChangeFont( aFont );
}

// --- MyFontDialog::CharModifyHdl() -------------------------------

void MyFontDialog::CharModifyHdl( ComboBox* pBox )
{
    aFont.ChangeCharOrientation( pBox->GetText() );
    aShowFont.ChangeFont( aFont );
}

// --- AppWin::AppWin() --------------------------------------------

AppWin::AppWin() :
            WorkWindow( NULL, WB_APP | WB_MOVEABLE |
                              WB_MINABLE | WB_CLOSEABLE ),
            aFontDialog( this ),
            aFontButton( this, WB_DEFBUTTON )
{
    aFontButton.SetText( "Change Font..." );
    aFontButton.ChangePosPixel( Point( 300, 10 ) );
    aFontButton.ChangeSizePixel( Size( 150, 30 ) );
    aFontButton.ChangeClickHdl( LINK( this, AppWin, FontHdl ) );
    aFontButton.Show();

    ChangeOutputSizePixel( Size( 500, 420 ) );
    SetText( "Font Demo" );
    Show();
}

// --- AppWin::Paint() ---------------------------------------------

void AppWin::Paint( const Rectangle& )
{
    String  aStr;
    Font    aTempFont;
    aTempFont.ChangePitch( PITCH_FIXED );
    aTempFont.ChangeSize( Size( 0, 14 ) );
    aTempFont.ChangeTransparent( TRUE );
    ChangeFont( aTempFont );

    Font aNewFont = aFontDialog.GetNewFont();
    FontMetric aMetric = GetFontMetric( aNewFont );
    Font       aMetricFont = aMetric.GetPhysicalFont();

    aStr  = "FontName       : ";
    aStr += aMetricFont.GetName();
    DrawText( Point( 10,  10 ), aStr );

    String aTempStr;
    switch ( aMetricFont.GetFamily() )
    {
        case FAMILY_SWISS:
            aTempStr = "Swiss";
            break;
        case FAMILY_MODERN:
            aTempStr = "Modern";
            break;
        case FAMILY_DECORATIVE:
            aTempStr = "Decorative";
            break;
        case FAMILY_ROMAN:
            aTempStr = "Roman";
            break;
        case FAMILY_SCRIPT:
            aTempStr = "Script";
            break;
        case FAMILY_SYSTEM:
            aTempStr = "System";
            break;
        default:
            aTempStr = "DontKnow";
            break;
    }
    aStr  = "Family         : ";
    aStr += aTempStr;
    DrawText( Point( 10,  30 ), aStr );

    aStr  = "FontSize       : ";
    aStr += aMetricFont.GetSize().Width();
    aStr += ';';
    aStr += aMetricFont.GetSize().Height();
    DrawText( Point( 10,  50 ), aStr );

    switch ( aMetricFont.GetPitch() )
    {
        case PITCH_FIXED:
            aTempStr = "Fixed";
            break;
        case PITCH_VARIABLE:
            aTempStr = "Variable";
            break;
        default:
            aTempStr = "DontKnow";
            break;
    }
    aStr  = "Pitch          : ";
    aStr += aTempStr;
    DrawText( Point( 10,  70 ), aStr );

    switch ( aMetricFont.GetCharSet() )
    {
        case CHARSET_ANSI:
            aTempStr = "Ansi";
            break;
        case CHARSET_IBMPC_850:
            aTempStr = "IBM-PC";
            break;
        case CHARSET_MAC:
            aTempStr = "Mac";
            break;
        case CHARSET_SYMBOL:
            aTempStr = "Symbol";
            break;
        default:
            aTempStr = "DontKnow";
            break;
    }
    aStr  = "CharSet        : ";
    aStr += aTempStr;
    DrawText( Point( 10,  90 ), aStr );

    switch ( aMetric.GetType() )
    {
        case TYPE_RASTER:
            aTempStr = "Raster";
            break;
        case TYPE_VECTOR:
            aTempStr = "Vector";
            break;
        case TYPE_SCALABLE:
            aTempStr = "Scalable";
            break;
        default:
            aTempStr = "DontKnow";
            break;
    }
    aStr  = "FontType       : ";
    aStr += aTempStr;
    DrawText( Point( 10, 110 ), aStr );

    switch ( aMetricFont.GetWeight() )
    {
        case WEIGHT_LIGHT:
            aTempStr = "Light";
            break;
        case WEIGHT_NORMAL:
            aTempStr = "Normal";
            break;
        case WEIGHT_BOLD:
            aTempStr = "Bold";
            break;
        default:
            aTempStr = "DontKnow";
            break;
    }
    aStr  = "Weight         : ";
    aStr += aTempStr;
    DrawText( Point( 10, 130 ), aStr );

    aStr  = "Italic         : ";
    aStr += (aMetricFont.IsItalic()) ? "TRUE" : "FALSE";
    DrawText( Point( 10, 150 ), aStr );

    switch ( aMetricFont.GetUnderline() )
    {
        case UNDERLINE_NONE:
            aTempStr = "UNDERLINE_NONE";
            break;
        case UNDERLINE_SINGLE:
            aTempStr = "UNDERLINE_SINGLE";
            break;
        case UNDERLINE_DOUBLE:
            aTempStr = "UNDERLINE_DOUBLE";
            break;
    }
    aStr  = "Underline      : ";
    aStr += aTempStr;
    DrawText( Point( 10, 170 ), aStr );

    switch ( aMetricFont.GetStrikeout() )
    {
        case STRIKEOUT_NONE:
            aTempStr = "STRIKEOUT_NONE";
            break;
        case STRIKEOUT_SINGLE:
            aTempStr = "STRIKEOUT_SINGLE";
            break;
        case STRIKEOUT_DOUBLE:
            aTempStr = "STRIKEOUT_DOUBLE";
            break;
    }
    aStr  = "Strikeout      : ";
    aStr += aTempStr;
    DrawText( Point( 10, 190 ), aStr );

    aStr  = "Shadow         : ";
    aStr += (aMetricFont.IsShadow()) ? "TRUE" : "FALSE";
    DrawText( Point( 10, 210 ), aStr );
    aStr  = "Outline        : ";
    aStr += (aMetricFont.IsOutline()) ? "TRUE" : "FALSE";
    DrawText( Point( 10, 230 ), aStr );
    aStr  = "Transparent    : ";
    aStr += (aMetricFont.IsTransparent()) ? "TRUE" : "FALSE";
    DrawText( Point( 10, 250 ), aStr );

    aStr  = "IsDeviceFont   : ";
    aStr += (aMetric.IsDeviceFont()) ? "TRUE" : "FALSE";
    DrawText( Point( 10, 270 ), aStr );

    aStr  = "Ascent         : ";
    aStr += aMetric.GetAscent();
    DrawText( Point( 10, 290 ), aStr );
    aStr  = "Descent        : ";
    aStr += aMetric.GetDescent();
    DrawText( Point( 10, 310 ), aStr );
    aStr  = "Leading        : " ;
    aStr += aMetric.GetLeading();
    DrawText( Point( 10, 330 ), aStr );
    aStr  = "Slant          : ";
    aStr += aMetric.GetSlant();
    DrawText( Point( 10, 350 ), aStr );

    aStr  = "LineOrientation: ";
    aStr += aMetricFont.GetLineOrientation();
    DrawText( Point( 10, 370 ), aStr );
    aStr  = "CharOrientation: ";
    aStr += aMetricFont.GetCharOrientation();
    DrawText( Point( 10, 390 ), aStr );

    ChangeFont( aNewFont );
    DrawText( Point( 300, 200 ), "Test" );
}

// --- AppWin::FontHdl() -------------------------------------------

void AppWin::FontHdl( PushButton* )
{
    if ( aFontDialog.Execute() )
        Invalidate();
}

// --- MyApp::Main() -----------------------------------------------

void MyApp::Main( int, char*[] )
{
    EnableSVLook();

    AppWin aAppWin;
    Execute();
}

// --- aMyApp ------------------------------------------------------

MyApp aMyApp;
