/*******************************************************************
*  PAINTDLG.CXX
*  (c) 1992-1994 STAR DIVISION
*******************************************************************/

#include <sv.hxx>

#include "paintdlg.hxx"
#include "paint.hrc"

// --- PenDialog::PenDialog() --------------------------------------

PenDialog::PenDialog( Window* pParent ) :
               ModalDialog( pParent, ResId( DLG_PENDIALOG ) ),
               aColorText( this, ResId( PD_COLORTEXT ) ),
               aColorList( this, ResId( PD_COLORLIST ) ),
               aWidthText( this, ResId( PD_WIDTHTEXT ) ),
               aWidthField( this, ResId( PD_WIDTHFIELD ) ),
               aStyleText( this, ResId( PD_STYLETEXT ) ),
               aStyleList( this, ResId( PD_STYLELIST ) ),
               aOKButton( this, ResId( PD_OK ) ),
               aCancelButton( this, ResId( PD_CANCEL ) )
{
    aOKButton.ChangeClickHdl( LINK( this, PenDialog, OKHdl ) );

    FreeResource();
}

// --- PenDialog::ChangeChoosePen() --------------------------------

Pen PenDialog::ChangeChoosePen( const Pen& rPen )
{
    Pen aOldPen = aChoosePen;

    aChoosePen = rPen;

    USHORT n;
    switch ( aChoosePen.GetColor().GetColorName() )
    {
        case COL_BLACK:
            n = 0;
            break;
        case COL_BLUE:
            n = 1;
            break;
        case COL_GREEN:
            n = 2;
            break;
        case COL_CYAN:
            n = 3;
            break;
        case COL_RED:
            n = 4;
            break;
        case COL_MAGENTA:
            n = 5;
            break;
        case COL_BROWN:
            n = 6;
            break;
        case COL_GRAY:
            n = 7;
            break;
        case COL_LIGHTGRAY:
            n = 8;
            break;
        case COL_LIGHTBLUE:
            n = 9;
            break;
        case COL_LIGHTGREEN:
            n = 10;
            break;
        case COL_LIGHTCYAN:
            n = 11;
            break;
        case COL_LIGHTRED:
            n = 12;
            break;
        case COL_LIGHTMAGENTA:
            n = 13;
            break;
        case COL_YELLOW:
            n = 14;
            break;
        case COL_WHITE:
            n = 15;
            break;
    }
    aColorList.SelectEntryPos( n );

    aWidthField.SetValue( aChoosePen.GetWidth() );

    switch ( aChoosePen.GetStyle() )
    {
        case PEN_NULL:
            n = 0;
            break;
        case PEN_SOLID:
            n = 1;
            break;
        case PEN_DOT:
            n = 2;
            break;
        case PEN_DASH:
            n = 3;
            break;
        case PEN_DASHDOT:
            n = 4;
            break;
    }
    aStyleList.SelectEntryPos( n );

    return aOldPen;
}

// --- PenDialog::OKHdl() ------------------------------------------

void PenDialog::OKHdl( OKButton* )
{
    switch ( aColorList.GetSelectEntryPos() )
    {
        case 0:
            aChoosePen.ChangeColor( Color( COL_BLACK ) );
            break;
        case 1:
            aChoosePen.ChangeColor( Color( COL_BLUE ) );
            break;
        case 2:
            aChoosePen.ChangeColor( Color( COL_GREEN ) );
            break;
        case 3:
            aChoosePen.ChangeColor( Color( COL_CYAN ) );
            break;
        case 4:
            aChoosePen.ChangeColor( Color( COL_RED ) );
            break;
        case 5:
            aChoosePen.ChangeColor( Color( COL_MAGENTA ) );
            break;
        case 6:
            aChoosePen.ChangeColor( Color( COL_BROWN ) );
            break;
        case 7:
            aChoosePen.ChangeColor( Color( COL_GRAY ) );
            break;
        case 8:
            aChoosePen.ChangeColor( Color( COL_LIGHTGRAY ) );
            break;
        case 9:
            aChoosePen.ChangeColor( Color( COL_LIGHTBLUE ) );
            break;
        case 10:
            aChoosePen.ChangeColor( Color( COL_LIGHTGREEN ) );
            break;
        case 11:
            aChoosePen.ChangeColor( Color( COL_LIGHTCYAN ) );
            break;
        case 12:
            aChoosePen.ChangeColor( Color( COL_LIGHTRED ) );
            break;
        case 13:
            aChoosePen.ChangeColor( Color( COL_LIGHTMAGENTA ) );
            break;
        case 14:
            aChoosePen.ChangeColor( Color( COL_YELLOW ) );
            break;
        case 15:
            aChoosePen.ChangeColor( Color( COL_WHITE ) );
            break;
    }

    aChoosePen.ChangeWidth( (USHORT)aWidthField.GetValue() );

    switch ( aStyleList.GetSelectEntryPos() )
    {
        case 0:
            aChoosePen.ChangeStyle( PEN_NULL );
            break;
        case 1:
            aChoosePen.ChangeStyle( PEN_SOLID );
            break;
        case 2:
            aChoosePen.ChangeStyle( PEN_DOT );
            break;
        case 3:
            aChoosePen.ChangeStyle( PEN_DASH );
            break;
        case 4:
            aChoosePen.ChangeStyle( PEN_DASHDOT );
            break;
    }

    EndDialog( TRUE );
}

// --- BrushDialog::BrushDialog() ----------------------------------

BrushDialog::BrushDialog( Window* pParent ) :
                 ModalDialog( pParent, ResId( DLG_BRUSHDIALOG ) ),
                 aColorText( this, ResId( BD_COLORTEXT ) ),
                 aColorList( this, ResId( BD_COLORLIST ) ),
                 aFillColorText( this, ResId( BD_FILLCOLORTEXT ) ),
                 aFillColorList( this, ResId( BD_FILLCOLORLIST ) ),
                 aStyleText( this, ResId( BD_STYLETEXT ) ),
                 aStyleList( this, ResId( BD_STYLELIST ) ),
                 aTransparentBox( this, ResId( BD_TRANSPARENTBOX ) ),
                 aOKButton( this, ResId( BD_OK ) ),
                 aCancelButton( this, ResId( BD_CANCEL ) )
{
    aOKButton.ChangeClickHdl( LINK( this, BrushDialog, OKHdl ) );

    FreeResource();
}

// --- BrushDialog::ChangeChooseBrush() ----------------------------

Brush BrushDialog::ChangeChooseBrush( const Brush& rBrush )
{
    Brush aOldBrush = aChooseBrush;

    aChooseBrush = rBrush;

    USHORT n;
    switch ( aChooseBrush.GetColor().GetColorName() )
    {
        case COL_BLACK:
            n = 0;
            break;
        case COL_BLUE:
            n = 1;
            break;
        case COL_GREEN:
            n = 2;
            break;
        case COL_CYAN:
            n = 3;
            break;
        case COL_RED:
            n = 4;
            break;
        case COL_MAGENTA:
            n = 5;
            break;
        case COL_BROWN:
            n = 6;
            break;
        case COL_GRAY:
            n = 7;
            break;
        case COL_LIGHTGRAY:
            n = 8;
            break;
        case COL_LIGHTBLUE:
            n = 9;
            break;
        case COL_LIGHTGREEN:
            n = 10;
            break;
        case COL_LIGHTCYAN:
            n = 11;
            break;
        case COL_LIGHTRED:
            n = 12;
            break;
        case COL_LIGHTMAGENTA:
            n = 13;
            break;
        case COL_YELLOW:
            n = 14;
            break;
        case COL_WHITE:
            n = 15;
            break;
    }
    aColorList.SelectEntryPos( n );

    switch ( aChooseBrush.GetFillColor().GetColorName() )
    {
        case COL_BLACK:
            n = 0;
            break;
        case COL_BLUE:
            n = 1;
            break;
        case COL_GREEN:
            n = 2;
            break;
        case COL_CYAN:
            n = 3;
            break;
        case COL_RED:
            n = 4;
            break;
        case COL_MAGENTA:
            n = 5;
            break;
        case COL_BROWN:
            n = 6;
            break;
        case COL_GRAY:
            n = 7;
            break;
        case COL_LIGHTGRAY:
            n = 8;
            break;
        case COL_LIGHTBLUE:
            n = 9;
            break;
        case COL_LIGHTGREEN:
            n = 10;
            break;
        case COL_LIGHTCYAN:
            n = 11;
            break;
        case COL_LIGHTRED:
            n = 12;
            break;
        case COL_LIGHTMAGENTA:
            n = 13;
            break;
        case COL_YELLOW:
            n = 14;
            break;
        case COL_WHITE:
            n = 15;
            break;
    }
    aFillColorList.SelectEntryPos( n );

    switch ( aChooseBrush.GetStyle() )
    {
        case BRUSH_NULL:
            n = 0;
            break;
        case BRUSH_SOLID:
            n = 1;
            break;
        case BRUSH_HORZ:
            n = 2;
            break;
        case BRUSH_VERT:
            n = 3;
            break;
        case BRUSH_CROSS:
            n = 4;
            break;
        case BRUSH_DIAGCROSS:
            n = 5;
            break;
        case BRUSH_UPDIAG:
            n = 6;
            break;
        case BRUSH_DOWNDIAG:
            n = 7;
            break;
        case BRUSH_25:
            n = 8;
            break;
        case BRUSH_50:
            n = 9;
            break;
        case BRUSH_75:
            n = 10;
            break;
    }
    aStyleList.SelectEntryPos( n );

    aTransparentBox.Check( aChooseBrush.IsTransparent() );

    return aOldBrush;
}

// --- BrushDialog::OKHdl() ----------------------------------------

void BrushDialog::OKHdl( OKButton* )
{
    switch ( aColorList.GetSelectEntryPos() )
    {
        case 0:
            aChooseBrush.ChangeColor( Color( COL_BLACK ) );
            break;
        case 1:
            aChooseBrush.ChangeColor( Color( COL_BLUE ) );
            break;
        case 2:
            aChooseBrush.ChangeColor( Color( COL_GREEN ) );
            break;
        case 3:
            aChooseBrush.ChangeColor( Color( COL_CYAN ) );
            break;
        case 4:
            aChooseBrush.ChangeColor( Color( COL_RED ) );
            break;
        case 5:
            aChooseBrush.ChangeColor( Color( COL_MAGENTA ) );
            break;
        case 6:
            aChooseBrush.ChangeColor( Color( COL_BROWN ) );
            break;
        case 7:
            aChooseBrush.ChangeColor( Color( COL_GRAY ) );
            break;
        case 8:
            aChooseBrush.ChangeColor( Color( COL_LIGHTGRAY ) );
            break;
        case 9:
            aChooseBrush.ChangeColor( Color( COL_LIGHTBLUE ) );
            break;
        case 10:
            aChooseBrush.ChangeColor( Color( COL_LIGHTGREEN ) );
            break;
        case 11:
            aChooseBrush.ChangeColor( Color( COL_LIGHTCYAN ) );
            break;
        case 12:
            aChooseBrush.ChangeColor( Color( COL_LIGHTRED ) );
            break;
        case 13:
            aChooseBrush.ChangeColor( Color( COL_LIGHTMAGENTA ) );
            break;
        case 14:
            aChooseBrush.ChangeColor( Color( COL_YELLOW ) );
            break;
        case 15:
            aChooseBrush.ChangeColor( Color( COL_WHITE ) );
            break;
    }

    switch ( aFillColorList.GetSelectEntryPos() )
    {
        case 0:
            aChooseBrush.ChangeFillColor( Color( COL_BLACK ) );
            break;
        case 1:
            aChooseBrush.ChangeFillColor( Color( COL_BLUE ) );
            break;
        case 2:
            aChooseBrush.ChangeFillColor( Color( COL_GREEN ) );
            break;
        case 3:
            aChooseBrush.ChangeFillColor( Color( COL_CYAN ) );
            break;
        case 4:
            aChooseBrush.ChangeFillColor( Color( COL_RED ) );
            break;
        case 5:
            aChooseBrush.ChangeFillColor( Color( COL_MAGENTA ) );
            break;
        case 6:
            aChooseBrush.ChangeFillColor( Color( COL_BROWN ) );
            break;
        case 7:
            aChooseBrush.ChangeFillColor( Color( COL_GRAY ) );
            break;
        case 8:
            aChooseBrush.ChangeFillColor( Color( COL_LIGHTGRAY ) );
            break;
        case 9:
            aChooseBrush.ChangeFillColor( Color( COL_LIGHTBLUE ) );
            break;
        case 10:
            aChooseBrush.ChangeFillColor( Color( COL_LIGHTGREEN ) );
            break;
        case 11:
            aChooseBrush.ChangeFillColor( Color( COL_LIGHTCYAN ) );
            break;
        case 12:
            aChooseBrush.ChangeFillColor( Color( COL_LIGHTRED ) );
            break;
        case 13:
            aChooseBrush.ChangeFillColor( Color( COL_LIGHTMAGENTA ) );
            break;
        case 14:
            aChooseBrush.ChangeFillColor( Color( COL_YELLOW ) );
            break;
        case 15:
            aChooseBrush.ChangeFillColor( Color( COL_WHITE ) );
            break;
    }

    switch ( aStyleList.GetSelectEntryPos() )
    {
        case 0:
            aChooseBrush.ChangeStyle( BRUSH_NULL );
            break;
        case 1:
            aChooseBrush.ChangeStyle( BRUSH_SOLID );
            break;
        case 2:
            aChooseBrush.ChangeStyle( BRUSH_HORZ );
            break;
        case 3:
            aChooseBrush.ChangeStyle( BRUSH_VERT );
            break;
        case 4:
            aChooseBrush.ChangeStyle( BRUSH_CROSS );
            break;
        case 5:
            aChooseBrush.ChangeStyle( BRUSH_DIAGCROSS );
            break;
        case 6:
            aChooseBrush.ChangeStyle( BRUSH_UPDIAG );
            break;
        case 7:
            aChooseBrush.ChangeStyle( BRUSH_DOWNDIAG );
            break;
        case 8:
            aChooseBrush.ChangeStyle( BRUSH_25 );
            break;
        case 9:
            aChooseBrush.ChangeStyle( BRUSH_50 );
            break;
        case 10:
            aChooseBrush.ChangeStyle( BRUSH_75 );
            break;
    }

    aChooseBrush.ChangeTransparent( aTransparentBox.IsChecked() );

    EndDialog( TRUE );
}
