/*******************************************************************
*  PAINTWIN.CXX
*  (c) 1992-1994 STAR DIVISION
*******************************************************************/

#include <iostream.h>
#include <fstream.h>

#include <sv.hxx>

#include "paintwin.hxx"
#include "paintdlg.hxx"
#include "paint.hxx"
#include "paint.hrc"

// --- PaintDocWin::PaintDocWin() ----------------------------------

PaintDocWin::PaintDocWin() :
                MDIWindow( pApp->GetAppWindow(), WB_STDMDI ),
                aActPen( Color( (ColorName)COL_BLACK ), 1 ),
                aActBrush( Color( (ColorName)COL_WHITE ),
                           (BrushStyle)BRUSH_NULL ),
                aActFont( System::GetSystemFont() ),
                aActText( "StarView" ),
                aDragPen( Color( (ColorName)COL_WHITE ), 0 ),
                aDragBrush( (BrushStyle)BRUSH_NULL ),
                aFileName( ResId( STRING_UNTITLED ) )
{
    Init();

    aMtf.Record( this );
    ChangePen( aActPen );
    ChangeFillInBrush( aActBrush );
    ChangeFont( aActFont );

    SetText( aFileName );
    Show();
}

// --- PaintDocWin::PaintDocWin() ----------------------------------

PaintDocWin::PaintDocWin( const String& rName ) :
                MDIWindow( pApp->GetAppWindow(), WB_STDMDI ),
                aActPen( Color( (ColorName)COL_BLACK ), 1 ),
                aActBrush( Color( (ColorName)COL_WHITE ),
                           (BrushStyle)BRUSH_NULL ),
                aActFont( System::GetSystemFont() ),
                aActText( "StarView" ),
                aDragPen( Color( (ColorName)COL_WHITE ), 0 ),
                aDragBrush( (BrushStyle)BRUSH_NULL ),
                aFileName( rName )
{
    Init();

    Load();

    aMtf.Record( this );
    ChangePen( aActPen );
    ChangeFillInBrush( aActBrush );
    ChangeFont( aActFont );

    SetText( aFileName );
    Show();
}

// --- PaintDocWin::~PaintDocWin() ---------------------------------

PaintDocWin::~PaintDocWin()
{
    Hide();

    PaintApp* pPaintApp = (PaintApp*)pApp;
    if ( pPaintApp->GetWindowCount() == 1 )
        pPaintApp->SetInitMenu();
}

// --- PaintDocWin::MouseButtonDown() ------------------------------

void PaintDocWin::MouseButtonDown( const MouseEvent& rMEvt )
{
    Update();

    if ( !rMEvt.IsLeft() )
        return;

    Point aPosition = PixelToLogic( rMEvt.GetPosPixel() );

    switch ( nActDraw )
    {
        case MI_DRAWPOINT:
            DrawPoint( aPosition );
            break;

        case MI_DRAWLINE:
            bDrag = TRUE;
            aMtf.Pause( TRUE );
            CaptureMouse();
            aDragStart = aOldDrag = aPosition;
            ChangeRasterOp( ROP_XOR );
            ChangePen( aDragPen );
            DrawLine( aDragStart, aOldDrag );
            break;

        case MI_DRAWRECT:
            {
            bDrag = TRUE;
            aMtf.Pause( TRUE );
            CaptureMouse();
            aDragStart = aOldDrag = aPosition;
            ChangeRasterOp( ROP_XOR );
            ChangePen( aDragPen );
            ChangeFillInBrush( aDragBrush );
            DrawRect( Rectangle( aDragStart, aOldDrag ) );
            }
            break;

        case MI_DRAWELLIPSE:
            {
            bDrag = TRUE;
            aMtf.Pause( TRUE );
            CaptureMouse();
            aDragStart = aOldDrag = aPosition;
            ChangeRasterOp( ROP_XOR );
            ChangePen( aDragPen );
            ChangeFillInBrush( aDragBrush );
            DrawEllipse( Rectangle( aDragStart, aOldDrag ) );
            }
            break;

        case MI_DRAWARC:
            {
            bDrag = TRUE;
            aMtf.Pause( TRUE );
            CaptureMouse();
            aDragStart = aOldDrag = aPosition;
            ChangeRasterOp( ROP_XOR );
            ChangePen( aDragPen );
            ChangeFillInBrush( aDragBrush );
            Rectangle aRect( aDragStart, aOldDrag );
            Point aCenter = aRect.Center();
            DrawArc( aRect,
                     Point( aRect.Right(), aCenter.Y() ),
                     Point( aCenter.X(), aRect.Top() ) );
            }
            break;

        case MI_DRAWPIE:
            {
            bDrag = TRUE;
            aMtf.Pause( TRUE );
            CaptureMouse();
            aDragStart = aOldDrag = aPosition;
            ChangeRasterOp( ROP_XOR );
            ChangePen( aDragPen );
            ChangeFillInBrush( aDragBrush );
            Rectangle aRect( aDragStart, aOldDrag );
            Point aCenter = aRect.Center();
            DrawPie( aRect,
                     Point( aRect.Right(), aCenter.Y() ),
                     Point( aCenter.X(), aRect.Top() ) );
            }
            break;

        case MI_DRAWPOLYGON:
            {
            bDrag = TRUE;
            aMtf.Pause( TRUE );
            CaptureMouse();
            if ( !bPoly )
            {
                aPolygon.ChangeSize( 1 );
                aPolygon.ChangePoint( aPosition, 0 );
                aOldDrag = aDragStart = aPosition;
                bPoly = TRUE;
                ChangeRasterOp( ROP_XOR );
                ChangePen( aDragPen );
                DrawLine( aDragStart, aOldDrag );
            }
            }
            break;

        case MI_DRAWTEXT:
            {
            DrawText( aPosition, aActText );
            }
            break;

        case MI_DRAWINVERTRECT:
            {
            bDrag = TRUE;
            aMtf.Pause( TRUE );
            CaptureMouse();
            aDragStart = aOldDrag = aPosition;
            ChangeRasterOp( ROP_XOR );
            ChangePen( aDragPen );
            ChangeFillInBrush( aDragBrush );
            DrawRect( Rectangle( aDragStart, aOldDrag ) );
            }
            break;
    }
}

// --- PaintDocWin::MouseButtonUp() --------------------------------

void PaintDocWin::MouseButtonUp( const MouseEvent& rMEvt )
{
    if ( !rMEvt.IsLeft() || (!bDrag && !bPoly) )
        return;

    Point aPosition = PixelToLogic( rMEvt.GetPosPixel() );

    switch ( nActDraw )
    {
        case MI_DRAWLINE:
            ReleaseMouse();
            DrawLine( aDragStart, aOldDrag );
            ChangeRasterOp( ROP_OVERPAINT );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            aMtf.Pause( FALSE );
            DrawLine( aDragStart, aPosition );
            break;

        case MI_DRAWRECT:
            {
            ReleaseMouse();
            DrawRect( Rectangle( aDragStart, aOldDrag ) );
            ChangeRasterOp( ROP_OVERPAINT );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            aMtf.Pause( FALSE );
            DrawRect( Rectangle( aDragStart, aPosition ) );
            }
            break;

        case MI_DRAWELLIPSE:
            {
            ReleaseMouse();
            DrawEllipse( Rectangle( aDragStart, aOldDrag ) );
            ChangeRasterOp( ROP_OVERPAINT );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            aMtf.Pause( FALSE );
            DrawEllipse( Rectangle( aDragStart, aPosition ) );
            }
            break;

        case MI_DRAWARC:
            {
            ReleaseMouse();
            Rectangle aOldRect( aDragStart, aOldDrag );
            Rectangle aNewRect( aDragStart, aPosition );
            Point aOldCenter = aOldRect.Center();
            Point aNewCenter = aNewRect.Center();

            DrawArc( aOldRect,
                     Point( aOldRect.Right(), aOldCenter.Y() ),
                     Point( aOldCenter.X(), aOldRect.Top() ) );

            ChangeRasterOp( ROP_OVERPAINT );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            aMtf.Pause( FALSE );

            DrawArc( aNewRect,
                     Point( aNewRect.Right(), aNewCenter.Y() ),
                     Point( aNewCenter.X(), aNewRect.Top() ) );
            }
            break;

        case MI_DRAWPIE:
            {
            ReleaseMouse();
            Rectangle aOldRect( aDragStart, aOldDrag );
            Rectangle aNewRect( aDragStart, aPosition );
            Point aOldCenter = aOldRect.Center();
            Point aNewCenter = aNewRect.Center();

            DrawPie( aOldRect,
                     Point( aOldRect.Right(), aOldCenter.Y() ),
                     Point( aOldCenter.X(), aOldRect.Top() ) );

            ChangeRasterOp( ROP_OVERPAINT );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            aMtf.Pause( FALSE );

            DrawPie( aNewRect,
                     Point( aNewRect.Right(), aNewCenter.Y() ),
                     Point( aNewCenter.X(), aNewRect.Top() ) );
            }
            break;

        case MI_DRAWPOLYGON:
            {
            DrawLine( aDragStart, aOldDrag );
            if ( aPolygon.GetSize() > 1 )
                DrawLine( aOldDrag, aPolygon.GetPoint( 0 ) );
            ChangeRasterOp( ROP_OVERPAINT );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            if ( rMEvt.GetClicks() == 1 )
            {
                DrawLine( aDragStart, aPosition );
                if ( aPosition != aDragStart)
                {
                    aPolygon.ChangeSize( aPolygon.GetSize()+1 );
                    aPolygon.ChangePoint( aPosition,
                                          aPolygon.GetSize()-1 );
                    aDragStart = aOldDrag = aPosition;
                }
                ChangeRasterOp( ROP_XOR );
                ChangePen( aDragPen );
                DrawLine( aDragStart, aOldDrag );
                if ( aPolygon.GetSize() > 1 )
                    DrawLine( aOldDrag, aPolygon.GetPoint( 0 ) );
            }
            else
            {
                bPoly = FALSE;
                ReleaseMouse();
                aMtf.Pause( FALSE );
                DrawPolygon( aPolygon );
                bDrag = FALSE;
            }
            }
            break;

        case MI_DRAWINVERTRECT:
            {
            ReleaseMouse();
            DrawRect( Rectangle( aDragStart, aOldDrag ) );
            ChangeRasterOp( ROP_OVERPAINT );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            aMtf.Pause( FALSE );
            InvertRect( Rectangle( aDragStart, aPosition ) );
            }
            break;
    }

    bDrag = FALSE;
}

// --- PaintDocWin::MouseMove() ------------------------------------

void PaintDocWin::MouseMove( const MouseEvent& rMEvt )
{
    if ( !bDrag )
        return;

    Point aPosition = PixelToLogic( rMEvt.GetPosPixel() );

    switch ( nActDraw )
    {
        case MI_DRAWLINE:
            DrawLine( aDragStart, aOldDrag );
            DrawLine( aDragStart, aPosition );
            aOldDrag = aPosition;
            break;

        case MI_DRAWRECT:
            DrawRect( Rectangle( aDragStart, aOldDrag ) );
            DrawRect( Rectangle( aDragStart, aPosition ) );
            aOldDrag = aPosition;
            break;

        case MI_DRAWELLIPSE:
            DrawEllipse( Rectangle( aDragStart, aOldDrag ) );
            DrawEllipse( Rectangle( aDragStart, aPosition ) );
            aOldDrag = aPosition;
            break;

        case MI_DRAWARC:
            {
            Rectangle aOldRect( aDragStart, aOldDrag );
            Rectangle aNewRect( aDragStart, aPosition );
            Point aOldCenter = aOldRect.Center();
            Point aNewCenter = aNewRect.Center();
            aOldDrag = aPosition;
            DrawArc( aOldRect,
                     Point( aOldRect.Right(), aOldCenter.Y() ),
                     Point( aOldCenter.X(), aOldRect.Top() ) );
            DrawArc( aNewRect,
                     Point( aNewRect.Right(), aNewCenter.Y() ),
                     Point( aNewCenter.X(), aNewRect.Top() ) );
            aOldDrag = aPosition;
            }
            break;

        case MI_DRAWPIE:
            {
            Rectangle aOldRect( aDragStart, aOldDrag );
            Rectangle aNewRect( aDragStart, aPosition );
            Point aOldCenter = aOldRect.Center();
            Point aNewCenter = aNewRect.Center();
            aOldDrag = aPosition;
            DrawPie( aOldRect,
                     Point( aOldRect.Right(), aOldCenter.Y() ),
                     Point( aOldCenter.X(), aOldRect.Top() ) );
            DrawPie( aNewRect,
                     Point( aNewRect.Right(), aNewCenter.Y() ),
                     Point( aNewCenter.X(), aNewRect.Top() ) );
            aOldDrag = aPosition;
            }
            break;

        case MI_DRAWPOLYGON:
            DrawLine( aDragStart, aOldDrag );
            if ( aPolygon.GetSize() > 1 )
                DrawLine( aOldDrag, aPolygon.GetPoint( 0 ) );
            aOldDrag = aPosition;
            DrawLine( aDragStart, aOldDrag );
            if ( aPolygon.GetSize() > 1 )
                DrawLine( aOldDrag, aPolygon.GetPoint( 0 ) );
            break;

        case MI_DRAWINVERTRECT:
            DrawRect( Rectangle( aDragStart, aOldDrag ) );
            DrawRect( Rectangle( aDragStart, aPosition ) );
            aOldDrag = aPosition;
            break;
    }
}

// --- PaintDocWin::Paint() ----------------------------------------

void PaintDocWin::Paint( const Rectangle& )
{
    aMtf.Stop();
    aMtf.WindStart();

    Push();
    Size aSize = System::GetScreenSizePixel();
    aMtf.Play( this, Point(),
               PixelToLogic( aSize, MapMode( MAP_10TH_MM ) ) );
    Pop();

    aMtf.Record( this );
}

// --- PaintDocWin::Close() ----------------------------------------

BOOL PaintDocWin::Close()
{
    delete this;
    return TRUE;
}

// --- PaintDocWin::Activate() -------------------------------------

void PaintDocWin::Activate()
{
    if ( !IsMDIActivate() )
        return;

    MenuBar* pMenu = pApp->GetAppMenu();
    pMenu->GetPopupMenu( MENU_VIEW )->CheckItem( nActView, TRUE );
    pMenu->GetPopupMenu( MENU_DRAW )->CheckItem( nActDraw, TRUE );

    ToolBox* pBox  = ((PaintAppWin*)pApp->GetAppWindow())->GetToolBox();
    pBox->CheckItem( nActDraw, TRUE );
}

// --- PaintDocWin::Deactivate() -----------------------------------

void PaintDocWin::Deactivate()
{
    if ( !IsMDIActivate() )
        return;

    MenuBar* pMenu = pApp->GetAppMenu();
    pMenu->GetPopupMenu( MENU_VIEW )->CheckItem( nActView, FALSE );
    pMenu->GetPopupMenu( MENU_DRAW )->CheckItem( nActDraw, FALSE );

    ToolBox* pBox  = ((PaintAppWin*)pApp->GetAppWindow())->GetToolBox();
    pBox->CheckItem( nActDraw, FALSE );
}

// --- PaintDocWin::Init() -----------------------------------------

void PaintDocWin::Init()
{
    ChangeIcon( Icon( ResId( ICON_WORK ) ) );

    ((PaintApp*)pApp)->SetAppMenu();

    ChangeMapMode( MapMode( MAP_10TH_MM ) );

    aMtf.ChangePrefMapMode( GetMapMode() );
    aMtf.ChangePrefSize( PixelToLogic( System::GetScreenSizePixel() ) );

    aActFont.ChangeColor( Color( COL_BLACK ) );
    aActFont.ChangeTransparent( TRUE );
    aActFont.ChangePitch( PITCH_DONTKNOW );
    aActFont.ChangeFamily( FAMILY_DONTKNOW );
    aActFont.ChangeCharSet( CHARSET_DONTKNOW );
    aActFont.ChangeSize( PixelToLogic( aActFont.GetSize() ) );

    nActView = MI_VIEW100;
    nActDraw = MI_DRAWPOINT;
    bDrag    = FALSE;
    bPoly    = FALSE;
}

// --- PaintDocWin::Load() -----------------------------------------

void PaintDocWin::Load()
{
#if defined( __ZTC__ ) || defined( UNX ) || defined( MAC )
    ifstream aStream( (const char*)aFileName, ios::in );
#else
    ifstream aStream( (const char*)aFileName, ios::in | ios::binary );
#endif
    if ( aStream.good() )
        aStream >> aMtf;
}

// --- PaintDocWin::Save() -----------------------------------------

void PaintDocWin::Save()
{
    if ( aFileName == String( ResId( STRING_UNTITLED ) ) )
    {
        SaveAs();
        return;
    }

#if defined( __ZTC__ ) || defined( UNX ) || defined( MAC )
    ofstream aStream( (const char*)aFileName, ios::out );
#else
    ofstream aStream( (const char*)aFileName, ios::out | ios::binary );
#endif
    if ( aStream.good() )
        aStream << aMtf;
}

// --- PaintDocWin::SaveAs() ---------------------------------------

void PaintDocWin::SaveAs()
{
    FileDialog aDlg( this, WB_SAVEAS | WB_SVLOOK | WB_STDMODAL );

    if ( aDlg.Execute() )
    {
        aFileName = aDlg.GetPath();
        SetText( aFileName );
        Save();
    }
}

// --- PaintDocWin::Print() ----------------------------------------

void PaintDocWin::Print()
{
    PrinterSetupDialog aSetupDlg( this, WB_SVLOOK | WB_STDMODAL );
    PrintDialog        aDlg( this, WB_SVLOOK | WB_STDMODAL );

    aDlg.ChangePrinter( &aPrinter );
    aDlg.ChangeSetupDialog( &aSetupDlg );
    aDlg.ChangeCopyCount( 1 );
    aDlg.ChangeFirstPage( 1 );
    aDlg.ChangeLastPage( 1 );
    aDlg.EnablePageFields( FALSE );
    aDlg.EnableSelection( FALSE );
    aDlg.EnableCollate( FALSE );

    if ( aDlg.Execute() )
    {
        aPrinter.SetCopyCount( aDlg.GetCopyCount() );

        if ( !aPrinter.StartJob( aFileName ) )
        {
            ErrorBox( this, ResId( DLG_PRINTERROR1 ) ).Execute();
            return;
        }

        aPrinter.StartPage();

        aMtf.Stop();
        aMtf.WindStart();
        aMtf.Play( &aPrinter, Point(), aPrinter.GetOutputSizePixel() );
        aMtf.Record( this );

        aPrinter.EndPage();

        if ( !aPrinter.EndJob() )
        {
            ErrorBox( this, ResId( DLG_PRINTERROR2 ) ).Execute();
        }
    }
}

// --- PaintDocWin::PrintSetup() -----------------------------------

void PaintDocWin::PrintSetup()
{
    PrinterSetupDialog aDlg( this, WB_SVLOOK | WB_STDMODAL );
    aDlg.ChangePrinter( &aPrinter );
    aDlg.Execute();
}

// --- PaintDocWin::View() -----------------------------------------

void PaintDocWin::View( USHORT nItemId )
{
    Fraction aFract;

    switch ( nItemId )
    {
        case MI_VIEW50:
            {
            aFract = Fraction( 1, 2 );
            }
            break;
        case MI_VIEW100:
            {
            aFract = Fraction( 1, 1 );
            }
            break;
        case MI_VIEW200:
            {
            aFract = Fraction( 2, 1 );
            }
            break;
        case MI_VIEW400:
            {
            aFract = Fraction( 4, 1 );
            }
            break;
        case MI_VIEW600:
            {
            aFract = Fraction( 6, 1 );
            }
            break;
    }

    PopupMenu* pMenu = pApp->GetAppMenu()->GetPopupMenu( MENU_VIEW );

    MapMode aMapMode = GetMapMode();
    aMapMode.ChangeScaleX( aFract );
    aMapMode.ChangeScaleY( aFract );
    aMtf.Pause( TRUE );
    ChangeMapMode( aMapMode );
    aMtf.Pause( FALSE );
    pMenu->CheckItem( nActView, FALSE );
    nActView = nItemId;
    pMenu->CheckItem( nActView, TRUE );
    Invalidate();
}

// --- PaintDocWin::Command() --------------------------------------

BOOL PaintDocWin::Command( USHORT nItemId )
{
    switch ( nItemId )
    {
        case MI_FILESAVE:
            Save();
            break;
        case MI_FILESAVEAS:
            SaveAs();
            break;
        case MI_FILECLOSE:
            Close();
            break;
        case MI_FILEPRINT:
            Print();
            break;
        case MI_FILESETUP:
            PrintSetup();
            break;

        case MI_EDITCUT:
            Clipboard::Clear();
            Clipboard::CopyGDIMetaFile( aMtf );
            aMtf.Stop();
            aMtf.Clear();
            aMtf.Record( this );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            ChangeFont( aActFont );
            Invalidate();
            break;
        case MI_EDITCOPY:
            Clipboard::Clear();
            Clipboard::CopyGDIMetaFile( aMtf );
            break;
        case MI_EDITPASTE:
            {
            aMtf.Stop();
            Clipboard::PasteGDIMetaFile( aMtf );
            aMtf.Record( this );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            ChangeFont( aActFont );
            Invalidate();
            }
            break;
        case MI_EDITCLEAR:
            aMtf.Stop();
            aMtf.Clear();
            aMtf.Record( this );
            ChangePen( aActPen );
            ChangeFillInBrush( aActBrush );
            ChangeFont( aActFont );
            Invalidate();
            break;

        case MI_VIEW50:
        case MI_VIEW100:
        case MI_VIEW200:
        case MI_VIEW400:
        case MI_VIEW600:
            View( nItemId );
            break;

        case MI_DRAWPOINT:
        case MI_DRAWLINE:
        case MI_DRAWRECT:
        case MI_DRAWELLIPSE:
        case MI_DRAWARC:
        case MI_DRAWPIE:
        case MI_DRAWPOLYGON:
        case MI_DRAWTEXT:
        case MI_DRAWINVERTRECT:
            {
            PopupMenu* pMenu = pApp->GetAppMenu()->GetPopupMenu( MENU_DRAW );
            ToolBox*   pBox  = ((PaintAppWin*)pApp->GetAppWindow())->
                                    GetToolBox();
            pMenu->CheckItem( nActDraw, FALSE );
            pBox->CheckItem( nActDraw, FALSE );
            nActDraw = nItemId;
            pMenu->CheckItem( nActDraw, TRUE );
            pBox->CheckItem( nActDraw, TRUE );
            }
            break;

        case MI_OPTIONPEN:
            {
            PenDialog aDialog( this );
            aDialog.ChangeChoosePen( aActPen );
            if ( aDialog.Execute() )
            {
                aActPen = aDialog.GetChoosePen();
                ChangePen( aActPen );
            }
            }
            break;
        case MI_OPTIONBRUSH:
            {
            BrushDialog aDialog( this );
            aDialog.ChangeChooseBrush( aActBrush );
            if ( aDialog.Execute() )
            {
                aActBrush = aDialog.GetChooseBrush();
                ChangeFillInBrush( aActBrush );
            }
            }
            break;
        case MI_OPTIONTEXT:
            {
            FontDialog aDialog( this, WB_SVLOOK | WB_STDMODAL );
            Font       aTempFont = aActFont;
            Size       aSize = aTempFont.GetSize();
            aSize = LogicToPixel( aSize );
            aSize = PixelToLogic( aSize, MAP_POINT );
            aSize.Width()  *= 10;
            aSize.Height() *= 10;
            aTempFont.ChangeSize( aSize );
            aDialog.ChangeFont( aTempFont );
            if ( aDialog.Execute() )
            {
                aTempFont = aDialog.GetFont();
                Size aSize = aTempFont.GetSize();
                aSize.Width()  /= 10;
                aSize.Height() /= 10;
                aSize = LogicToPixel( aSize, MAP_POINT );
                aSize = PixelToLogic( aSize );
                aTempFont.ChangeSize( aSize );
                aActFont = aTempFont;
                ChangeFont( aActFont );
            }
            }
            break;
        case MI_OPTIONBACKGROUND:
            {
            BrushDialog aDialog( this );
            aDialog.ChangeChooseBrush( GetBackgroundBrush() );
            if ( aDialog.Execute() )
            {
                aMtf.Pause( TRUE );
                ChangeBackgroundBrush( aDialog.GetChooseBrush() );
                aMtf.Pause( FALSE );
                Invalidate();
            }
            }
            break;

        default:
            return FALSE;
    }

    return TRUE;
}
